@@ -1,3 +1,5 @@
+from __future__ import absolute_import
+
 """
 M2Crypto is the most complete Python wrapper for OpenSSL featuring RSA, DSA,
 DH, EC, HMACs, message digests, symmetric ciphers (including AES); SSL
@@ -15,36 +17,25 @@ Copyright (C) 2004-2007 OSAF. All Rights Reserved.
 
 Copyright 2008-2011 Heikki Toivonen. All rights reserved.
 """
+# noqa
+from distutils.version import StrictVersion
+__version__ = '0.25.1'
+version = __version__  # type: str
+version_info = StrictVersion(__version__).version
 
-version_info = (0, 22)
-version = '.'.join([str(_v) for _v in version_info])
+from M2Crypto import (ASN1, AuthCookie, BIO, BN, DH, DSA, EVP, Engine, Err,
+                      PGP, RC4, RSA, Rand, SMIME, SSL, X509, _m2crypto,
+                      ftpslib, httpslib, m2, m2urllib, m2xmlrpclib,
+                      threading, util)
+if util.py27plus:
+    from typing import Tuple  # noqa
 
-import __m2crypto
-import m2
-import ASN1
-import AuthCookie
-import BIO
-import BN
-import Rand
-import DH
-import DSA
 if m2.OPENSSL_VERSION_NUMBER >= 0x90800F and m2.OPENSSL_NO_EC == 0:
-    import EC
-import Err
-import Engine
-import EVP
-import RSA
-import RC4
-import SMIME
-import SSL
-import X509
+    from M2Crypto import EC
+# Backwards compatibility.
+urllib2 = m2urllib
 
-import m2urllib
-import m2urllib2
-import ftpslib
-import httpslib
-import m2xmlrpclib
-import threading
-import util
+encrypt = 1
+decrypt = 0
 
-__m2crypto.lib_init()
+_m2crypto.lib_init()
