@@ -1,3 +1,5 @@
+from __future__ import absolute_import
+
 """
 M2Crypto enhancement to Python's urllib2 for handling
 'https' url's.
@@ -12,16 +14,46 @@ Summary of changes:
 """
 
 import socket
-import urlparse
 
-import SSL
+from M2Crypto import SSL, httpslib, six, util
 
-import httpslib
-from urllib2 import *  # noqa
+from M2Crypto.six.moves.urllib_parse import urldefrag, urlparse as url_parse
+from M2Crypto.six.moves.urllib_response import addinfourl
+if util.py27plus:
+    from typing import List, Optional  # noqa
+
+# six.moves doesn't support star imports
+if six.PY3:
+    from urllib.request import *  # noqa other modules want to import
+    from urllib.error import *  # noqa other modules want to import
+else:
+    from urllib2 import *  # noqa
+
+
+try:
+    mother_class = socket._fileobject
+except AttributeError:
+    mother_class = socket.SocketIO
+
+
+class _closing_fileobject(mother_class):  # noqa
+    '''socket._fileobject that propagates self.close() to the socket.
+
+    Python 2.5 provides this as socket._fileobject(sock, close=True).
+    '''
+
+# for python 3
+try:
+    AbstractHTTPHandler
+except NameError:
+    # somehow this won't get imported by the import * above
+    import urllib.request
+    AbstractHTTPHandler = urllib.request.AbstractHTTPHandler
 
 
 class HTTPSHandler(AbstractHTTPHandler):
     def __init__(self, ssl_context=None):
+        # type: (SSL.Context) -> None
         AbstractHTTPHandler.__init__(self)
 
         if ssl_context is not None:
@@ -32,6 +64,7 @@ class HTTPSHandler(AbstractHTTPHandler):
 
     # Copied from urllib2, so we can set the ssl context.
     def https_open(self, req):
+        # type: (Request) -> addinfourl
         """Return an addinfourl object for the request, using http_class.
 
         http_class must implement the HTTPConnection API from httplib.
@@ -41,20 +74,27 @@ class HTTPSHandler(AbstractHTTPHandler):
             - geturl(): return the original request URL
             - code: HTTP status code
         """
-        host = req.get_host()
+        # https://docs.python.org/3.3/library/urllib.request.html#urllib.request.Request.get_host
+        try:     # up to python-3.2
+            host = req.get_host()
+        except AttributeError:  # from python-3.3
+            host = req.host
         if not host:
             raise URLError('no host given')
 
         # Our change: Check to see if we're using a proxy.
         # Then create an appropriate ssl-aware connection.
         full_url = req.get_full_url()
-        target_host = urlparse.urlparse(full_url)[1]
+        target_host = url_parse(full_url)[1]
 
         if (target_host != host):
-            request_uri = urlparse.urldefrag(full_url)[0]
+            request_uri = urldefrag(full_url)[0]
             h = httpslib.ProxyHTTPSConnection(host=host, ssl_context=self.ctx)
         else:
-            request_uri = req.get_selector()
+            try:     # up to python-3.2
+                request_uri = req.get_selector()
+            except AttributeError:  # from python-3.3
+                request_uri = req.selector
             h = httpslib.HTTPSConnection(host=host, ssl_context=self.ctx)
         # End our change
         h.set_debuglevel(self._debuglevel)
@@ -95,6 +135,7 @@ class HTTPSHandler(AbstractHTTPHandler):
 
 # Copied from urllib2 with modifications for ssl
 def build_opener(ssl_context=None, *handlers):
+    # type: (Optional[SSL.Context], *List[object]) -> OpenerDirector
     """Create an opener object from a list of handlers.
 
     The opener will use several default handlers, including support
@@ -103,10 +144,9 @@ def build_opener(ssl_context=None, *handlers):
     If any of the handlers passed as arguments are subclasses of the
     default handlers, the default handlers will not be used.
     """
-    import types
 
     def isclass(obj):
-        return isinstance(obj, types.ClassType) or hasattr(obj, "__bases__")
+        return isinstance(obj, type) or hasattr(obj, "__bases__")
 
     opener = OpenerDirector()
     default_classes = [ProxyHandler, UnknownHandler, HTTPHandler,
